import { NextFunction, Request, Response } from "express";
import prisma from "../../database/dbclient";
import { z } from "zod";
import bcrypt from "bcryptjs";
import crypto from "crypto";

export const getAllJobByCompany = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id } = req.params;
  const { page = 1, limit = 12 } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const skip = (pageNum - 1) * limitNum;

  try {
    const users = await prisma.job_db.findMany({
      where: { company_id: parseInt(company_id) },
      include: {},
      take: limitNum,
      skip: skip,
    });

    const total_rows = await prisma.job_db.count({
      where: { company_id: Number.parseInt(company_id) },
    });
    const total_page = Math.ceil(total_rows / limitNum);
    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    next(error);
  }
};

export const getAllJobMetadataByCompany = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id } = req.params;
  console.log(
    "GET > Get all jobs metadata by company id > company id > ",
    company_id
  );

  try {
    const jobs = await prisma.job_db.findMany({
      where: { company_id: parseInt(company_id) },

      select: {
        id: true,
        title: true,
        department: {
          select: {
            dpt_name: true,
          },
        },
        designation: {
          select: {
            designation_name: true,
          },
        },
      },
    });

    res.status(200).json({
      data: jobs,
    });
  } catch (error) {
    console.error("GET > Get all jobs metadata by company id > ", error);
    next(error);
  }
};

// Zod schema for job creation
const createJobSchema = z.object({
  employee_id: z.number().nullable(),
  desc: z.string().max(5000),
  title: z.string(),
  company_id: z.number(),
  dept_id: z.number(),
  designation_id: z.number(),
  lastDate: z.string().refine((date) => !isNaN(Date.parse(date)), {
    message: "Invalid date format",
  }),
  status: z.number().optional(),
  jobCode: z.string(),
  isPublished: z.number().optional(),
});
// Zod schema for job update
const updateJobSchema = z.object({
  desc: z.string().max(5000).optional(),
  title: z.string().optional(),
  company_id: z.number().optional(),
  dept_id: z.number().optional(),
  designation_id: z.number().optional(),
  lastDate: z
    .string()
    .optional()
    .refine((date) => !isNaN(Date.parse(date ?? "")), {
      message: "Invalid date format",
    }),
  status: z.number().optional(),
  jobCode: z.string().optional(),
  isPublished: z.number().optional(),
});

export const createJob = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate input data
    const validatedData = createJobSchema.parse(req.body);

    // Create a new job
    const newJob = await prisma.job_db.create({
      data: {
        ...validatedData,
        lastDate: new Date(validatedData.lastDate),
      },
    });

    return res.status(201).json({ success: true, data: newJob });
  } catch (error) {
    console.error(
      "POST > Create a job listing > Failed to create a job listing > ",
      error
    );
    next(error);
  }
};

export const updateJob = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    const { job_id } = req.params;

    // Validate input data
    const validatedData = updateJobSchema.parse(req.body);

    // Update the job
    const updatedJob = await prisma.job_db.update({
      where: { id: parseInt(job_id) },
      data: {
        ...validatedData,
        lastDate: validatedData.lastDate
          ? new Date(validatedData.lastDate)
          : undefined,
      },
    });

    return res.status(200).json({ success: true, data: updatedJob });
  } catch (error) {
    next(error);
  }
};

export const getASingleJob = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    const { job_id } = req.params;

    // get a job
    const job = await prisma.job_db.findUnique({
      where: { id: parseInt(job_id) },
      include: {
        company: {
          select: {
            company_name: true,
            logo: true,
            contact_number: true,
            website: true,
            headquarters: true,
          },
        },
        department: {
          select: {
            dpt_name: true,
          },
        },
        designation: {
          select: {
            designation_name: true,
          },
        },
      },
    });

    return res.status(200).json({ success: true, data: job });
  } catch (error) {
    next(error);
  }
};

const createJobApplicantSchema = z.object({
  last_name: z.string(),
  middle_name: z.string().nullable(),
  first_name: z.string(),
  job_id: z.number(),
  email: z.string(),
  cv_url: z.string(),
  cover_letter_url: z.string().optional(),
  job_status: z.string().optional(),
  uni_key: z.string(),
  company_id: z.number(),
});

export const createJobApplicant = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate input data
    var validatedData = createJobApplicantSchema.parse(req.body);

    var singleJob = await prisma.job_db.findUnique({
      where: { id: validatedData.job_id },
    });

    if (singleJob?.status != 1) {
      return res.status(201).json({ success: true, message: "Job is closed" });
    }

    validatedData.uni_key = validatedData.job_id + validatedData.email;

    // Create a new job
    const newJob = await prisma.job_applicant_db.create({
      data: validatedData,
    });

    //TODO create a email here
    return res.status(201).json({ success: true, data: newJob });
  } catch (error) {
    next(error);
  }
};

export const getAllCandidateByCompany = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id } = req.params;
  const { page = 1, limit = 10, category } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const skip = (pageNum - 1) * limitNum;

  const hiredOnly = category === "hired";
  const rejectedOnly = category === "rejected";

  const statusFilters = [];
  if (hiredOnly) {
    statusFilters.push("onboarded");
    statusFilters.push("hired");
    statusFilters.push("accepted");
    // Add more statuses if required
  } else if (rejectedOnly) {
    statusFilters.push("rejected");
    statusFilters.push("denied");
    statusFilters.push("cancelled");
    // Add more statuses if required
  } else {
    statusFilters.push("pending");
  }

  try {
    const users = await prisma.job_applicant_db.findMany({
      where: {
        AND: {
          company_id: parseInt(company_id),
          job_status: { in: statusFilters },
        },
      },
      select: {
        id: true,
        job_id: true,
        first_name: true,
        middle_name: true,
        last_name: true,
        email: true,
        job_status: true,
        cv_url: true,
        cover_letter_url: true,
        job: {
          include: {
            department: true,
            designation: true,
          },
        },
      },
      take: limitNum,
      skip: skip,
    });

    const total_rows = await prisma.job_applicant_db.count({
      where: {
        AND: {
          company_id: parseInt(company_id),
          job_status: { in: statusFilters },
        },
      },
    });
    const total_page = Math.ceil(total_rows / limitNum);
    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    next(error);
  }
};

export const getAllCandidateByJobID = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { job_id } = req.params;
  const { page = 1, limit = 10, category } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const hiredOnly = category === "hired";
  const rejectedOnly = category === "rejected";

  const statusFilters = [];
  if (hiredOnly) {
    statusFilters.push("onboarded");
    statusFilters.push("hired");
    statusFilters.push("accepted");
    // Add more statuses if required
  } else if (rejectedOnly) {
    statusFilters.push("rejected");
    statusFilters.push("denied");
    statusFilters.push("cancelled");
    // Add more statuses if required
  } else {
    statusFilters.push("pending");
  }

  const skip = (pageNum - 1) * limitNum;

  try {
    const users = await prisma.job_applicant_db.findMany({
      where: {
        AND: {
          job_id: Number.parseInt(job_id),
          job_status: {
            in: statusFilters,
          },
        },
      },
      include: {
        job: {
          include: {
            department: true,
            designation: true,
          },
        },
      },
      take: limitNum,
      skip: skip,
    });

    const total_rows = await prisma.job_applicant_db.count({
      where: {
        AND: {
          job_id: Number.parseInt(job_id),
          job_status: {
            in: statusFilters,
          },
        },
      },
    });
    const total_page = Math.ceil(total_rows / limitNum);
    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    next(error);
  }
};

const UpdateobApplicantStatusSchema = z.object({
  job_id: z.number(),
  applicant_id: z.number(),
});

export const makeShortListApplicant = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate input data
    var validatedData = UpdateobApplicantStatusSchema.parse(req.body);

    var singleJob = await prisma.job_db.findUnique({
      where: { id: validatedData.job_id },
    });

    if (singleJob?.status != 1) {
      return res.status(401).json({ success: true, message: "Job is closed" });
    }

    // Create a new job
    const newJob = await prisma.job_applicant_db.update({
      where: {
        id: validatedData.applicant_id,
      },
      data: {
        job_status: "shortlisted",
      },
    });

    //TODO create a email here
    return res
      .status(201)
      .json({ success: true, data: "Applicant is Shortlisted" });
  } catch (error) {
    next(error);
  }
};

export const getAllShortListedCandidateByCompanyID = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id = "0" } = req.params;
  const { page = 1, limit = 10, job_id = 0 } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);
  const companyId = parseInt(company_id as string);
  const skip = (pageNum - 1) * limitNum;

  if (companyId == 0 || Number.isNaN(companyId)) {
    return res.status(400).json({
      message: "Invalid Company ID.",
    });
  }

  try {
    const users = await prisma.job_applicant_db.findMany({
      where: {
        AND: [
          {
            company_id: companyId,
          },
          {
            job_status: "shortlisted",
          },
        ],
      },
      take: limitNum,
      skip: skip,
      include: {
        job: {
          include: {
            department: true,
            designation: true,
          },
        },
      },
    });

    const total_rows = await prisma.job_applicant_db.count({
      where: {
        AND: [
          {
            company_id: companyId,
          },
          {
            job_status: "shortlisted",
          },
        ],
      },
    });
    const total_page = Math.ceil(total_rows / limitNum);
    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    next(error);
  }
};

export const getAllShortListedCandidateByJobID = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { page = 1, limit = 10, job_id = 0 } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);
  const jobId = parseInt(job_id as string);
  const skip = (pageNum - 1) * limitNum;

  if (jobId == 0 || Number.isNaN(jobId)) {
    return res.status(400).json({
      message: "Invalid Job ID.",
    });
  }

  try {
    const users = await prisma.job_applicant_db.findMany({
      where: {
        AND: [
          {
            job_id: jobId,
          },
          {
            job_status: "shortlisted",
          },
        ],
      },
      take: limitNum,
      skip: skip,
      include: {
        job: {
          include: {
            department: true,
            designation: true,
          },
        },
      },
    });

    const total_rows = await prisma.job_applicant_db.count({
      where: {
        AND: [
          {
            job_id: jobId,
          },
          {
            job_status: "shortlisted",
          },
        ],
      },
    });
    const total_page = Math.ceil(total_rows / limitNum);
    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    console.error("GET > Get shortlisted Applicants by Job ID > ", error);
    next(error);
  }
};

const OnBoardApplicantSchema = z.object({
  job_id: z.number(),
  applicant_id: z.number(),
});

export const OnBoardApplicant = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate input data
    var validatedData = OnBoardApplicantSchema.parse(req.body);

    var applicant = await prisma.job_applicant_db.findUnique({
      where: { id: validatedData.applicant_id },
    });

    var singleJob = await prisma.job_db.findUnique({
      where: { id: validatedData.job_id },
    });

    var user = await prisma.users.findUnique({
      where: { email: applicant?.email },
      include: {
        employee_data: true,
      },
    });

    if (user != null) {
      return res
        .status(401)
        .json({ success: true, message: "User Already Exist In The System." });
    }

    const password = await generateHashedPassword("123456");

    await prisma.$transaction(async (prisma) => {
      // Create the new user

      // Create the new user
      const newUser = await prisma.users.create({
        data: {
          email: applicant?.email ?? "",
          password_hash: password,
          first_name: applicant?.first_name ?? "",
          last_name: applicant?.last_name ?? "",
          middle_name: applicant?.middle_name ?? "",
        },
      });

      const employee = await prisma.employees.create({
        data: {
          user_id: newUser.user_id,
          company_id: singleJob?.company_id ?? 0,
          employee_code: generateUniqueEmployeeCode(
            applicant?.first_name ?? "",
            applicant?.last_name ?? ""
          ),
          designation_id: singleJob?.designation_id,
          department_id: singleJob?.dept_id,
        },
      });

      const companyRole = await prisma.usercompany.create({
        data: {
          user_id: newUser.user_id,
          company_id: singleJob?.company_id ?? 0,
          company_role_id: 4, // 4 means employee
        },
      });

      const user_roles = await prisma.user_roles.create({
        data: {
          user_id: newUser.user_id,
          role_id: 4, // 4 means employee
        },
      });

      const Updateapplicant = await prisma.job_applicant_db.update({
        where: { id: validatedData.applicant_id },
        data: {
          job_status: "onboarded",
        },
      });
    });

    //TODO create a email here
    return res.status(201).json({
      success: true,

      msg: "Applicant is Onboarded Successfully..",
      user: {
        email: applicant?.email,
      },
      password,
    });
  } catch (error) {
    next(error);
  }
};

const generateHashedPassword = async (
  plainTextPassword: string
): Promise<string> => {
  console.log(plainTextPassword);
  const saltRounds = 10; // Defines the complexity of the hash
  const hashedPassword = await bcrypt.hash(plainTextPassword, saltRounds);
  return hashedPassword;
};

function generateUniqueEmployeeCode(
  firstName: string,
  lastName: string
): string {
  // Get the first letter of first and last name
  const firstInitial = firstName.charAt(0).toUpperCase();
  const lastInitial = lastName.charAt(0).toUpperCase();

  // Generate a random number or use a timestamp for uniqueness
  const uniqueSuffix = Date.now().toString().slice(-4); // Using last 4 digits of the current timestamp

  // Construct the unique employee code
  const employeeCode = `${firstInitial}${lastInitial}-${uniqueSuffix}`;

  return employeeCode;
}
